@@ -664,6 +664,8 @@ xor_code_t* init_xor_hd_code(int k, int m, int hd)
       if (k <= 10 && k >= 5) {
         is_valid = 1;
       }
+    } else if (m == 3) {
+      is_valid = 1;
     }
   }
   
