@@ -30,10 +30,13 @@
 #include "erasurecode.h"
 #include "erasurecode_backend.h"
 #include "erasurecode_helpers.h"
+#include "erasurecode_helpers_ext.h"
 #include "erasurecode_preprocessing.h"
 #include "erasurecode_postprocessing.h"
 #include "erasurecode_stdinc.h"
+
 #include "alg_sig.h"
+#include "erasurecode_log.h"
 
 /* =~=*=~==~=*=~==~=*=~= Supported EC backends =~=*=~==~=*=~==~=*=~==~=*=~== */
 
@@ -253,6 +256,12 @@ int liberasurecode_instance_create(const ec_backend_id_t id,
     if (id >= EC_BACKENDS_MAX)
         return -EBACKENDNOTSUPP;
 
+    if ((args->k + args->m) > EC_MAX_FRAGMENTS) {
+        log_error("Total number of fragments (k + m) must be less than %d\n",
+                  EC_MAX_FRAGMENTS);
+        return -EINVALIDPARAMS;
+    }
+
     /* Allocate memory for ec_backend instance */
     instance = calloc(1, sizeof(*instance));
     if (NULL == instance)
