@@ -30,6 +30,7 @@
 #include <stdbool.h>
 #include "erasurecode.h"
 #include "erasurecode_helpers.h"
+#include "erasurecode_helpers_ext.h"
 #include "erasurecode_preprocessing.h"
 #include "erasurecode_backend.h"
 #include "alg_sig.h"
@@ -39,6 +40,7 @@
 #define JERASURE_RS_CAUCHY_BACKEND "jerasure_rs_cauchy"
 #define ISA_L_RS_VAND_BACKEND "isa_l_rs_vand"
 #define SHSS_BACKEND "shss"
+#define RS_VAND_BACKEND "liberasurecode_rs_vand"
 
 typedef void (*TEST_FUNC)();
 
@@ -57,13 +59,17 @@ struct ec_args null_args = {
     .ct = CHKSUM_NONE,
 };
 
+struct ec_args *null_test_args[] = { &null_args, NULL };
+
 struct ec_args flat_xor_hd_args = {
-    .k = 10,
-    .m = 6,
-    .hd = 4,
+    .k = 3,
+    .m = 3,
+    .hd = 3,
     .ct = CHKSUM_NONE,
 };
 
+struct ec_args *flat_xor_test_args[] = { &flat_xor_hd_args, NULL };
+
 struct ec_args jerasure_rs_vand_args = {
     .k = 10,
     .m = 4,
@@ -72,6 +78,35 @@ struct ec_args jerasure_rs_vand_args = {
     .ct = CHKSUM_NONE,
 };
 
+struct ec_args jerasure_rs_vand_44_args = {
+    .k = 4,
+    .m = 4,
+    .w = 16,
+    .hd = 5,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args jerasure_rs_vand_48_args = {
+    .k = 4,
+    .m = 8,
+    .w = 16,
+    .hd = 9,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args jerasure_rs_vand_1010_args = {
+    .k = 10,
+    .m = 10,
+    .w = 16,
+    .hd = 11,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args *jerasure_rs_vand_test_args[] = { &jerasure_rs_vand_args, 
+                                                 &jerasure_rs_vand_44_args, 
+                                                 &jerasure_rs_vand_1010_args, 
+                                                 &jerasure_rs_vand_48_args, 
+                                                 NULL };
 struct ec_args jerasure_rs_cauchy_args = {
     .k = 10,
     .m = 4,
@@ -80,6 +115,37 @@ struct ec_args jerasure_rs_cauchy_args = {
     .ct = CHKSUM_NONE,
 };
 
+struct ec_args jerasure_rs_cauchy_44_args = {
+    .k = 4,
+    .m = 4,
+    .w = 4,
+    .hd = 5,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args jerasure_rs_cauchy_48_args = {
+    .k = 4,
+    .m = 8,
+    .w = 8,
+    .hd = 9,
+    .ct = CHKSUM_NONE,
+};
+
+
+struct ec_args jerasure_rs_cauchy_1010_args = {
+    .k = 10,
+    .m = 10,
+    .w = 8,
+    .hd = 11,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args *jerasure_rs_cauchy_test_args[] = { &jerasure_rs_cauchy_args, 
+                                                   &jerasure_rs_cauchy_44_args, 
+                                                   &jerasure_rs_cauchy_48_args, 
+                                                   &jerasure_rs_cauchy_1010_args, 
+                                                   NULL };
+
 struct ec_args isa_l_args = {
     .k = 10,
     .m = 4,
@@ -87,6 +153,25 @@ struct ec_args isa_l_args = {
     .hd = 5,
 };
 
+struct ec_args isa_l_44_args = {
+    .k = 4,
+    .m = 4,
+    .w = 8,
+    .hd = 5,
+};
+
+struct ec_args isa_l_1010_args = {
+    .k = 10,
+    .m = 10,
+    .w = 8,
+    .hd = 11,
+};
+
+struct ec_args *isa_l_test_args[] = { &isa_l_args, 
+                                      &isa_l_44_args,
+                                      &isa_l_1010_args,
+                                      NULL };
+
 int priv = 128;
 struct ec_args shss_args = {
     .k = 6,
@@ -95,6 +180,85 @@ struct ec_args shss_args = {
     .priv_args2 = &priv,
 };
 
+struct ec_args *shss_test_args[] = { &shss_args, NULL };
+
+struct ec_args liberasurecode_rs_vand_args = {
+    .k = 10,
+    .m = 4,
+    .w = 16,
+    .hd = 5,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args liberasurecode_rs_vand_44_args = {
+    .k = 4,
+    .m = 4,
+    .w = 16,
+    .hd = 5,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args liberasurecode_rs_vand_48_args = {
+    .k = 4,
+    .m = 8,
+    .w = 16,
+    .hd = 9,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args liberasurecode_rs_vand_1010_args = {
+    .k = 10,
+    .m = 10,
+    .w = 16,
+    .hd = 11,
+    .ct = CHKSUM_NONE,
+};
+
+struct ec_args *liberasurecode_rs_vand_test_args[] = {
+               &liberasurecode_rs_vand_args,
+               &liberasurecode_rs_vand_44_args,
+               &liberasurecode_rs_vand_1010_args,
+               &liberasurecode_rs_vand_48_args,
+               NULL };
+
+struct ec_args **all_backend_tests[] = {
+               null_test_args,
+               flat_xor_test_args,
+               jerasure_rs_vand_test_args,
+               jerasure_rs_cauchy_test_args,
+               isa_l_test_args,
+               shss_test_args,
+               liberasurecode_rs_vand_test_args,
+               NULL};
+
+int num_backends()
+{
+  int i = 0;
+
+  while (NULL != all_backend_tests[i]) {
+    i++;
+  }
+
+  return i;
+}
+
+int max_tests_for_backends()
+{
+  int n_backends = num_backends();
+  int i = 0;
+  int max = 0;
+
+  for (i = 0; i < n_backends; i++) {
+    int j = 0;
+    while (NULL != all_backend_tests[i][j]) {
+      j++;
+    }
+    if (j > max) {
+      max = j;
+    }
+  }
+  return max;
+}
 
 typedef enum {
     LIBEC_VERSION_MISMATCH,
@@ -117,37 +281,58 @@ char * get_name_from_backend_id(ec_backend_id_t be) {
             return ISA_L_RS_VAND_BACKEND;
         case EC_BACKEND_SHSS:
             return SHSS_BACKEND;
+        case EC_BACKEND_LIBERASURECODE_RS_VAND:
+            return RS_VAND_BACKEND;
         default:
             return "UNKNOWN";
     }
 }
 
-struct ec_args *create_ec_args(ec_backend_id_t be, ec_checksum_type_t ct)
+struct ec_args *create_ec_args(ec_backend_id_t be, ec_checksum_type_t ct, int backend_test_idx)
 {
     size_t ec_args_size = sizeof(struct ec_args);
     struct ec_args *template = NULL;
+    struct ec_args** backend_args_array = NULL;
+    int i = 0;
+
     switch(be) {
         case EC_BACKEND_NULL:
-            template = &null_args;
+            backend_args_array = null_test_args;
             break;
         case EC_BACKEND_JERASURE_RS_VAND:
-            template = &jerasure_rs_vand_args;
+            backend_args_array = jerasure_rs_vand_test_args;
             break;
         case EC_BACKEND_JERASURE_RS_CAUCHY:
-            template = &jerasure_rs_cauchy_args;
+            backend_args_array = jerasure_rs_cauchy_test_args;
+            break;
+        case EC_BACKEND_LIBERASURECODE_RS_VAND:
+            backend_args_array = liberasurecode_rs_vand_test_args;
             break;
         case EC_BACKEND_FLAT_XOR_HD:
-            template = &flat_xor_hd_args;
+            backend_args_array = flat_xor_test_args;
             break;
         case EC_BACKEND_ISA_L_RS_VAND:
-            template = &isa_l_args;
+            backend_args_array = isa_l_test_args;
             break;
         case EC_BACKEND_SHSS:
-            template = &shss_args;
+            backend_args_array = shss_test_args;
             break;
         default:
             return NULL;
     }
+
+    while (NULL != backend_args_array && NULL != backend_args_array[i]) {
+        if (i == backend_test_idx) {
+            template = backend_args_array[i];
+            break;
+        }
+        i++;
+    }
+
+    if (NULL == template) {
+      return NULL;
+    }
+
     struct ec_args *args = malloc(ec_args_size);
     assert(args);
     memcpy(args, template, ec_args_size);
@@ -177,6 +362,31 @@ int *create_skips_array(struct ec_args *args, int skip)
     return buf;
 }
 
+static int create_fake_frags_no_meta(char ***array, int num_frags,
+                                     const char *data, int data_len)
+{
+    int _num_frags = 0;
+    int i = 0;
+    char **ptr = NULL;
+
+    *array = malloc(num_frags * sizeof(char *));
+    if (array == NULL) {
+        _num_frags = -1;
+        goto out;
+    }
+
+    // add data and parity frags
+    ptr = *array;
+    for (i = 0; i < num_frags; i++) {
+        *ptr = (char *) malloc(data_len);
+        strncpy(*ptr++, data, data_len);
+        _num_frags++;
+    }
+
+out:
+    return _num_frags;
+}
+
 static int create_frags_array(char ***array,
                               char **data,
                               char **parity,
@@ -278,6 +488,15 @@ static void test_create_backend_invalid_args()
         fprintf (stderr, "Backend library not available!\n");
         return;
     }
+    struct ec_args invalid_args = {
+        .k = 100,
+        .m = 100,
+    };
+    desc = liberasurecode_instance_create(EC_BACKEND_NULL, &invalid_args);
+    if (-EBACKENDNOTAVAIL == desc) {
+        fprintf (stderr, "Backend library not available!\n");
+        return;
+    }
     assert(desc < 0);
 }
 
@@ -391,6 +610,7 @@ static void test_decode_invalid_args()
     int *skips = create_skips_array(&null_args, -1);
     char *decoded_data = NULL;
     uint64_t decoded_data_len = 0;
+    const char *fake_data = " ";
 
     desc = liberasurecode_instance_create(EC_BACKEND_NULL, &null_args);
     if (-EBACKENDNOTAVAIL == desc) {
@@ -398,12 +618,40 @@ static void test_decode_invalid_args()
         return;
     }
     assert(desc > 0);
+
+    // test with invalid fragments (no metadata headers)
+    num_avail_frags = create_fake_frags_no_meta(&avail_frags, (null_args.k +
+                                                null_args.m),
+                                                fake_data, strlen(fake_data));
+    assert(num_avail_frags > 0);
+
+    rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
+                               strlen(fake_data), 1,
+                               &decoded_data, &decoded_data_len);
+    // force_metadata_checks results in EINSUFFFRAGS
+    assert(rc == -EINSUFFFRAGS);
+
+    rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
+                               strlen(fake_data), 0,
+                               &decoded_data, &decoded_data_len);
+    assert(rc == -EBADHEADER);
+
+    // test with num_fragments < (k)
+    num_avail_frags = create_fake_frags_no_meta(&avail_frags, (null_args.k - 1),
+                                                " ", 1);
+    assert(num_avail_frags > 0);
+    rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
+                               strlen(fake_data), 1,
+                               &decoded_data, &decoded_data_len);
+    assert(rc == -EINSUFFFRAGS);
+
     rc = liberasurecode_encode(desc, orig_data, orig_data_size,
             &encoded_data, &encoded_parity, &encoded_fragment_len);
-    assert(0 == rc);
+    assert(rc == 0);
 
     num_avail_frags = create_frags_array(&avail_frags, encoded_data,
                                          encoded_parity, &null_args, skips);
+    assert(num_avail_frags > 0);
 
     rc = liberasurecode_decode(-1, avail_frags, num_avail_frags,
                                encoded_fragment_len, 1,
@@ -667,8 +915,11 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
     if (-EBACKENDNOTAVAIL == desc) {
         fprintf (stderr, "Backend library not available!\n");
         return;
-    }
-    assert(desc > 0);
+    } else if ((args->k + args->m) > EC_MAX_FRAGMENTS) {
+        assert(-EINVALIDPARAMS == desc);
+        return;
+    } else
+        assert(desc > 0);
 
     orig_data = create_buffer(orig_data_size, 'x');
     assert(orig_data != NULL);
@@ -698,7 +949,7 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
 
     num_avail_frags = create_frags_array(&avail_frags, encoded_data,
                                          encoded_parity, args, skip);
-    assert(num_avail_frags != -1);
+    assert(num_avail_frags > 0);
     rc = liberasurecode_decode(desc, avail_frags, num_avail_frags,
                                encoded_fragment_len, 1,
                                &decoded_data, &decoded_data_len);
@@ -720,6 +971,17 @@ static void encode_decode_test_impl(const ec_backend_id_t be_id,
     free(avail_frags);
 }
 
+/**
+ * Note: this test will attempt to reconstruct a single fragment when
+ * one or more other fragments are missing (specified by skip).  
+ *
+ * For example, if skip is [0, 0, 0, 1, 0, 0] and we are reconstructing
+ * fragment 5, then it will test the reconstruction of fragment 5 when 3
+ * and 5 are assumed unavailable.
+ *
+ * We only mark at most 2 as unavailable, as we cannot guarantee every situation
+ * will be able to habndle 3 failures.
+ */
 static void reconstruct_test_impl(const ec_backend_id_t be_id,
                                  struct ec_args *args,
                                  int *skip)
@@ -748,15 +1010,24 @@ static void reconstruct_test_impl(const ec_backend_id_t be_id,
     rc = liberasurecode_encode(desc, orig_data, orig_data_size,
             &encoded_data, &encoded_parity, &encoded_fragment_len);
     assert(rc == 0);
-    num_avail_frags = create_frags_array(&avail_frags, encoded_data,
-                                             encoded_parity, args, skip);
     out = malloc(encoded_fragment_len);
     assert(out != NULL);
     for (i = 0; i < num_fragments; i++) {
-        if (skip[i] == 0) {
-            continue;
-        }
         char *cmp = NULL;
+        // If the current fragment was not chosen as fragments to skip,
+        // remove it and the chosen fragments to skip from the available list
+        // and reset its state
+        if (skip[i] == 0) {
+            skip[i] = 1;
+            num_avail_frags = create_frags_array(&avail_frags, encoded_data,
+                                             encoded_parity, args, skip);
+            skip[i] = 0;
+        // Do not reset the skip state if the fragment was chosen as a fragment
+        // to skip for this invocation of the test
+        } else {
+            num_avail_frags = create_frags_array(&avail_frags, encoded_data,
+                                             encoded_parity, args, skip);
+        }
         if (i < args->k) {
             cmp = encoded_data[i];
         }
@@ -1003,13 +1274,13 @@ static void test_decode_with_missing_parity(const ec_backend_id_t be_id,
 static void test_decode_with_missing_multi_data(const ec_backend_id_t be_id,
                                                 struct ec_args *args)
 {
-    int max_num_missing = args->hd - 1;
+    int max_num_missing = args->k <= (args->hd - 1) ? args->k : args->hd - 1;
     int i,j;
     for (i = 0; i < args->k - max_num_missing + 1; i++) {
         int *skip = create_skips_array(args,-1);
         assert(skip != NULL);
         for (j = i; j < i + max_num_missing; j++) {
-            skip[j]=1;
+            skip[j % args->k]=1;
         }
         encode_decode_test_impl(be_id, args, skip);
         free(skip);
@@ -1037,8 +1308,8 @@ static void test_decode_with_missing_multi_data_parity(
 {
     int i,j;
     int max_num_missing = args->hd - 1;
-    int start = args->k - max_num_missing + 1;
-    for (i = start; i < start + max_num_missing -1; i++) {
+    int end = (args->k + args->m) - max_num_missing + 1;
+    for (i = 0; i < end; i++) {
         int *skip = create_skips_array(args,-1);
         assert(skip != NULL);
         for (j = i; j < i + max_num_missing; j++) {
@@ -1058,6 +1329,21 @@ static void test_simple_encode_decode(const ec_backend_id_t be_id,
     free(skip);
 }
 
+static void test_simple_encode_decode_over32()
+{
+    struct ec_args over32_args = {
+        .k = 30,
+        .m = 20,
+    };
+
+    int *skip = create_skips_array(&over32_args, 1);
+    assert(skip != NULL);
+    // should return an error
+    encode_decode_test_impl(EC_BACKEND_JERASURE_RS_VAND,
+                            &over32_args, skip);
+    free(skip);
+}
+
 static void test_simple_reconstruct(const ec_backend_id_t be_id,
                                     struct ec_args *args)
 {
@@ -1336,6 +1622,10 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_be_ver_mismatch,
         EC_BACKEND_FLAT_XOR_HD, CHKSUM_CRC32,
         .skip = false},
+    {"test_simple_encode_decode_over32",
+        test_simple_encode_decode_over32,
+        EC_BACKEND_JERASURE_RS_VAND, CHKSUM_CRC32,
+        .skip = false},
     // Jerasure RS Vand backend tests
     {"create_and_destroy_backend",
         test_create_and_destroy_backend,
@@ -1572,35 +1862,96 @@ struct testcase testcases[] = {
         test_verify_stripe_metadata_be_ver_mismatch,
         EC_BACKEND_SHSS, CHKSUM_CRC32,
         .skip = false},
+    // Internal RS Vand backend tests
+    {"create_and_destroy_backend",
+        test_create_and_destroy_backend,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"simple_encode_liberasurecode_rs_vand",
+        test_simple_encode_decode,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_data_liberasurecode_rs_vand",
+        test_decode_with_missing_data,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_multi_data_liberasurecode_rs_vand",
+        test_decode_with_missing_multi_data,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"decode_with_missing_multi_parity_liberasurecode_rs_vand",
+        test_decode_with_missing_multi_parity,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"test_decode_with_missing_multi_data_parity_liberasurecode_rs_vand",
+        test_decode_with_missing_multi_data_parity,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"simple_reconstruct_liberasurecode_rs_vand",
+        test_simple_reconstruct,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"test_fragments_needed_liberasurecode_rs_vand",
+        test_fragments_needed,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"test_get_fragment_metadata_liberasurecode_rs_vand",
+        test_get_fragment_metadata,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_NONE,
+        .skip = false},
+    {"test_get_fragment_metadata_liberasurecode_rs_vand_crc32",
+        test_get_fragment_metadata,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata",
+        test_verify_stripe_metadata,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+        .skip = false},
+    {"test_verify_stripe_metadata_libec_mismatch",
+         test_verify_stripe_metadata_libec_mismatch,
+         EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+         .skip = false},
+    {"test_verify_stripe_metadata_magic_mismatch",
+          test_verify_stripe_metadata_magic_mismatch,
+          EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+         .skip = false},
+    {"test_verify_stripe_metadata_be_id_mismatch",
+         test_verify_stripe_metadata_be_id_mismatch,
+         EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+         .skip = false},
+    {"test_verify_stripe_metadata_be_ver_mismatch",
+        test_verify_stripe_metadata_be_ver_mismatch,
+        EC_BACKEND_LIBERASURECODE_RS_VAND, CHKSUM_CRC32,
+        .skip = false},
     { NULL, NULL, 0, 0, false },
 };
 
 int main(int argc, char **argv)
 {
-    int ii = 0, num_cases = 0;
+    int ii = 0, num_cases = 0, i = 0;
+    int max_backend_tests = max_tests_for_backends();
 
-    for (num_cases = 0; testcases[num_cases].description; num_cases++) {
-        /* Just counting */
-    }
-
-    printf("1..%d\n", num_cases);
-
-    for (ii = 0; testcases[ii].description != NULL; ++ii) {
-        const char *testname = get_name_from_backend_id(testcases[ii].be_id);
-        fflush(stdout);
-        if (testcases[ii].skip) {
-            fprintf(stdout, "ok # SKIP %d - %s: %s\n", ii + 1,
-                    testcases[ii].description,
-                    (testname) ? testname : "");
-            continue;
+    for (i = 0; i < max_backend_tests; i++) {
+		    for (ii = 0; testcases[ii].description != NULL; ++ii) {
+		        const char *testname = get_name_from_backend_id(testcases[ii].be_id);
+		        fflush(stdout);
+		        if (testcases[ii].skip) {
+		            fprintf(stdout, "ok # SKIP %d - %s: %s (idx=%d)\n", num_cases,
+		                    testcases[ii].description,
+		                    (testname) ? testname : "", i);
+		            continue;
+		        }
+		        struct ec_args *args = create_ec_args(testcases[ii].be_id, testcases[ii].ct, i);
+            if (NULL != args) {
+		            testcases[ii].function(testcases[ii].be_id, args);
+		            fprintf(stdout, "ok %d - %s: %s (idx=%d)\n", num_cases,
+		                    testcases[ii].description,
+		                    (testname) ? testname : "", i);
+		            fflush(stdout);
+		            free(args);
+		            num_cases++;
+            } 
         }
-        struct ec_args *args = create_ec_args(testcases[ii].be_id, testcases[ii].ct);
-        testcases[ii].function(testcases[ii].be_id, args);
-        fprintf(stdout, "ok %d - %s: %s\n", ii + 1,
-                testcases[ii].description,
-                (testname) ? testname : "");
-        fflush(stdout);
-        free(args);
     }
     return 0;
 }
